@@ -17,15 +17,14 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
+	"fmt"
 	"io"
+	"log/slog"
 	"math"
 	"net/http"
 	"net/url"
 	"time"
 
-	"github.com/go-kit/log"
-	"github.com/go-kit/log/level"
-	"github.com/pkg/errors"
 	"github.com/prometheus/common/model"
 )
 
@@ -36,14 +35,14 @@ const (
 
 // Client allows sending batches of Prometheus samples to OpenTSDB.
 type Client struct {
-	logger log.Logger
+	logger *slog.Logger
 
 	url     string
 	timeout time.Duration
 }
 
 // NewClient creates a new Client.
-func NewClient(logger log.Logger, url string, timeout time.Duration) *Client {
+func NewClient(logger *slog.Logger, url string, timeout time.Duration) *Client {
 	return &Client{
 		logger:  logger,
 		url:     url,
@@ -78,7 +77,7 @@ func (c *Client) Write(samples model.Samples) error {
 	for _, s := range samples {
 		v := float64(s.Value)
 		if math.IsNaN(v) || math.IsInf(v, 0) {
-			level.Debug(c.logger).Log("msg", "Cannot send value to OpenTSDB, skipping sample", "value", v, "sample", s)
+			c.logger.Debug("Cannot send value to OpenTSDB, skipping sample", "value", v, "sample", s)
 			continue
 		}
 		metric := TagValue(s.Metric[model.MetricNameLabel])
@@ -105,7 +104,7 @@ func (c *Client) Write(samples model.Samples) error {
 	ctx, cancel := context.WithTimeout(context.Background(), c.timeout)
 	defer cancel()
 
-	req, err := http.NewRequest("POST", u.String(), bytes.NewBuffer(buf))
+	req, err := http.NewRequest(http.MethodPost, u.String(), bytes.NewBuffer(buf))
 	if err != nil {
 		return err
 	}
@@ -136,7 +135,7 @@ func (c *Client) Write(samples model.Samples) error {
 	if err := json.Unmarshal(buf, &r); err != nil {
 		return err
 	}
-	return errors.Errorf("failed to write %d samples to OpenTSDB, %d succeeded", r["failed"], r["success"])
+	return fmt.Errorf("failed to write %d samples to OpenTSDB, %d succeeded", r["failed"], r["success"])
 }
 
 // Name identifies the client as an OpenTSDB client.
