@@ -17,14 +17,13 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"log/slog"
 	"math"
 	"path/filepath"
 	"sync"
 	"time"
 	"unicode/utf8"
 
-	"github.com/go-kit/log"
-	"github.com/go-kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/model"
 	"go.uber.org/atomic"
@@ -226,7 +225,7 @@ func (m *dbMetrics) Unregister() {
 // DB represents a WAL-only storage. It implements storage.DB.
 type DB struct {
 	mtx    sync.RWMutex
-	logger log.Logger
+	logger *slog.Logger
 	opts   *Options
 	rs     *remote.Storage
 
@@ -251,7 +250,7 @@ type DB struct {
 }
 
 // Open returns a new agent.DB in the given directory.
-func Open(l log.Logger, reg prometheus.Registerer, rs *remote.Storage, dir string, opts *Options) (*DB, error) {
+func Open(l *slog.Logger, reg prometheus.Registerer, rs *remote.Storage, dir string, opts *Options) (*DB, error) {
 	opts = validateOptions(opts)
 
 	locker, err := tsdbutil.NewDirLocker(dir, "agent", l, reg)
@@ -306,11 +305,11 @@ func Open(l log.Logger, reg prometheus.Registerer, rs *remote.Storage, dir strin
 	}
 
 	if err := db.replayWAL(); err != nil {
-		level.Warn(db.logger).Log("msg", "encountered WAL read error, attempting repair", "err", err)
+		db.logger.Warn("encountered WAL read error, attempting repair", "err", err)
 		if err := w.Repair(err); err != nil {
 			return nil, fmt.Errorf("repair corrupted WAL: %w", err)
 		}
-		level.Info(db.logger).Log("msg", "successfully repaired WAL")
+		db.logger.Info("successfully repaired WAL")
 	}
 
 	go db.run()
@@ -335,7 +334,7 @@ func validateOptions(opts *Options) *Options {
 		opts.WALCompression = wlog.CompressionNone
 	}
 
-	// Revert Stripesize to DefaultStripsize if Stripsize is either 0 or not a power of 2.
+	// Revert StripeSize to DefaultStripeSize if StripeSize is either 0 or not a power of 2.
 	if opts.StripeSize <= 0 || ((opts.StripeSize & (opts.StripeSize - 1)) != 0) {
 		opts.StripeSize = tsdb.DefaultStripeSize
 	}
@@ -359,7 +358,7 @@ func validateOptions(opts *Options) *Options {
 }
 
 func (db *DB) replayWAL() error {
-	level.Info(db.logger).Log("msg", "replaying WAL, this may take a while", "dir", db.wal.Dir())
+	db.logger.Info("replaying WAL, this may take a while", "dir", db.wal.Dir())
 	start := time.Now()
 
 	dir, startFrom, err := wlog.LastCheckpoint(db.wal.Dir())
@@ -376,7 +375,7 @@ func (db *DB) replayWAL() error {
 		}
 		defer func() {
 			if err := sr.Close(); err != nil {
-				level.Warn(db.logger).Log("msg", "error while closing the wal segments reader", "err", err)
+				db.logger.Warn("error while closing the wal segments reader", "err", err)
 			}
 		}()
 
@@ -386,7 +385,7 @@ func (db *DB) replayWAL() error {
 			return fmt.Errorf("backfill checkpoint: %w", err)
 		}
 		startFrom++
-		level.Info(db.logger).Log("msg", "WAL checkpoint loaded")
+		db.logger.Info("WAL checkpoint loaded")
 	}
 
 	// Find the last segment.
@@ -395,7 +394,7 @@ func (db *DB) replayWAL() error {
 		return fmt.Errorf("finding WAL segments: %w", err)
 	}
 
-	// Backfil segments from the most recent checkpoint onwards.
+	// Backfill segments from the most recent checkpoint onwards.
 	for i := startFrom; i <= last; i++ {
 		seg, err := wlog.OpenReadSegment(wlog.SegmentName(db.wal.Dir(), i))
 		if err != nil {
@@ -405,12 +404,12 @@ func (db *DB) replayWAL() error {
 		sr := wlog.NewSegmentBufReader(seg)
 		err = db.loadWAL(wlog.NewReader(sr), multiRef)
 		if err := sr.Close(); err != nil {
-			level.Warn(db.logger).Log("msg", "error while closing the wal segments reader", "err", err)
+			db.logger.Warn("error while closing the wal segments reader", "err", err)
 		}
 		if err != nil {
 			return err
 		}
-		level.Info(db.logger).Log("msg", "WAL segment loaded", "segment", i, "maxSegment", last)
+		db.logger.Info("WAL segment loaded", "segment", i, "maxSegment", last)
 	}
 
 	walReplayDuration := time.Since(start)
@@ -571,7 +570,7 @@ func (db *DB) loadWAL(r *wlog.Reader, multiRef map[chunks.HeadSeriesRef]chunks.H
 	}
 
 	if v := nonExistentSeriesRefs.Load(); v > 0 {
-		level.Warn(db.logger).Log("msg", "found sample referencing non-existing series", "skipped_series", v)
+		db.logger.Warn("found sample referencing non-existing series", "skipped_series", v)
 	}
 
 	db.nextRef.Store(uint64(lastRef))
@@ -616,9 +615,9 @@ Loop:
 				ts = maxTS
 			}
 
-			level.Debug(db.logger).Log("msg", "truncating the WAL", "ts", ts)
+			db.logger.Debug("truncating the WAL", "ts", ts)
 			if err := db.truncate(ts); err != nil {
-				level.Warn(db.logger).Log("msg", "failed to truncate WAL", "err", err)
+				db.logger.Warn("failed to truncate WAL", "err", err)
 			}
 		}
 	}
@@ -631,7 +630,7 @@ func (db *DB) truncate(mint int64) error {
 	start := time.Now()
 
 	db.gc(mint)
-	level.Info(db.logger).Log("msg", "series GC completed", "duration", time.Since(start))
+	db.logger.Info("series GC completed", "duration", time.Since(start))
 
 	first, last, err := wlog.Segments(db.wal.Dir())
 	if err != nil {
@@ -679,7 +678,7 @@ func (db *DB) truncate(mint int64) error {
 		// If truncating fails, we'll just try it again at the next checkpoint.
 		// Leftover segments will still just be ignored in the future if there's a
 		// checkpoint that supersedes them.
-		level.Error(db.logger).Log("msg", "truncating segments failed", "err", err)
+		db.logger.Error("truncating segments failed", "err", err)
 	}
 
 	// The checkpoint is written and segments before it are truncated, so we
@@ -696,13 +695,13 @@ func (db *DB) truncate(mint int64) error {
 		// Leftover old checkpoints do not cause problems down the line beyond
 		// occupying disk space. They will just be ignored since a newer checkpoint
 		// exists.
-		level.Error(db.logger).Log("msg", "delete old checkpoints", "err", err)
+		db.logger.Error("delete old checkpoints", "err", err)
 		db.metrics.checkpointDeleteFail.Inc()
 	}
 
 	db.metrics.walTruncateDuration.Observe(time.Since(start).Seconds())
 
-	level.Info(db.logger).Log("msg", "WAL checkpoint complete", "first", first, "last", last, "duration", time.Since(start))
+	db.logger.Info("WAL checkpoint complete", "first", first, "last", last, "duration", time.Since(start))
 	return nil
 }
 
@@ -764,6 +763,7 @@ func (db *DB) Close() error {
 
 type appender struct {
 	*DB
+	hints *storage.AppendOptions
 
 	pendingSeries          []record.RefSeries
 	pendingSamples         []record.RefSample
@@ -784,6 +784,10 @@ type appender struct {
 	floatHistogramSeries []*memSeries
 }
 
+func (a *appender) SetOptions(opts *storage.AppendOptions) {
+	a.hints = opts
+}
+
 func (a *appender) Append(ref storage.SeriesRef, l labels.Labels, t int64, v float64) (storage.SeriesRef, error) {
 	// series references and chunk references are identical for agent mode.
 	headRef := chunks.HeadSeriesRef(ref)
@@ -977,9 +981,134 @@ func (a *appender) UpdateMetadata(storage.SeriesRef, labels.Labels, metadata.Met
 	return 0, nil
 }
 
-func (a *appender) AppendCTZeroSample(storage.SeriesRef, labels.Labels, int64, int64) (storage.SeriesRef, error) {
-	// TODO(bwplotka): Wire metadata in the Agent's appender.
-	return 0, nil
+func (a *appender) AppendHistogramCTZeroSample(ref storage.SeriesRef, l labels.Labels, t, ct int64, h *histogram.Histogram, fh *histogram.FloatHistogram) (storage.SeriesRef, error) {
+	if h != nil {
+		if err := h.Validate(); err != nil {
+			return 0, err
+		}
+	}
+	if fh != nil {
+		if err := fh.Validate(); err != nil {
+			return 0, err
+		}
+	}
+	if ct >= t {
+		return 0, storage.ErrCTNewerThanSample
+	}
+
+	series := a.series.GetByID(chunks.HeadSeriesRef(ref))
+	if series == nil {
+		// Ensure no empty labels have gotten through.
+		l = l.WithoutEmpty()
+		if l.IsEmpty() {
+			return 0, fmt.Errorf("empty labelset: %w", tsdb.ErrInvalidSample)
+		}
+
+		if lbl, dup := l.HasDuplicateLabelNames(); dup {
+			return 0, fmt.Errorf(`label name "%s" is not unique: %w`, lbl, tsdb.ErrInvalidSample)
+		}
+
+		var created bool
+		series, created = a.getOrCreate(l)
+		if created {
+			a.pendingSeries = append(a.pendingSeries, record.RefSeries{
+				Ref:    series.ref,
+				Labels: l,
+			})
+			a.metrics.numActiveSeries.Inc()
+		}
+	}
+
+	series.Lock()
+	defer series.Unlock()
+
+	if ct <= a.minValidTime(series.lastTs) {
+		return 0, storage.ErrOutOfOrderCT
+	}
+
+	if ct > series.lastTs {
+		series.lastTs = ct
+	} else {
+		// discard the sample if it's out of order.
+		return 0, storage.ErrOutOfOrderCT
+	}
+
+	switch {
+	case h != nil:
+		zeroHistogram := &histogram.Histogram{}
+		a.pendingHistograms = append(a.pendingHistograms, record.RefHistogramSample{
+			Ref: series.ref,
+			T:   ct,
+			H:   zeroHistogram,
+		})
+		a.histogramSeries = append(a.histogramSeries, series)
+	case fh != nil:
+		a.pendingFloatHistograms = append(a.pendingFloatHistograms, record.RefFloatHistogramSample{
+			Ref: series.ref,
+			T:   ct,
+			FH:  &histogram.FloatHistogram{},
+		})
+		a.floatHistogramSeries = append(a.floatHistogramSeries, series)
+	}
+
+	a.metrics.totalAppendedSamples.WithLabelValues(sampleMetricTypeHistogram).Inc()
+	return storage.SeriesRef(series.ref), nil
+}
+
+func (a *appender) AppendCTZeroSample(ref storage.SeriesRef, l labels.Labels, t, ct int64) (storage.SeriesRef, error) {
+	if ct >= t {
+		return 0, storage.ErrCTNewerThanSample
+	}
+
+	series := a.series.GetByID(chunks.HeadSeriesRef(ref))
+	if series == nil {
+		l = l.WithoutEmpty()
+		if l.IsEmpty() {
+			return 0, fmt.Errorf("empty labelset: %w", tsdb.ErrInvalidSample)
+		}
+
+		if lbl, dup := l.HasDuplicateLabelNames(); dup {
+			return 0, fmt.Errorf(`label name "%s" is not unique: %w`, lbl, tsdb.ErrInvalidSample)
+		}
+
+		newSeries, created := a.getOrCreate(l)
+		if created {
+			a.pendingSeries = append(a.pendingSeries, record.RefSeries{
+				Ref:    newSeries.ref,
+				Labels: l,
+			})
+			a.metrics.numActiveSeries.Inc()
+		}
+
+		series = newSeries
+	}
+
+	series.Lock()
+	defer series.Unlock()
+
+	if t <= a.minValidTime(series.lastTs) {
+		a.metrics.totalOutOfOrderSamples.Inc()
+		return 0, storage.ErrOutOfOrderSample
+	}
+
+	if ct > series.lastTs {
+		series.lastTs = ct
+	} else {
+		// discard the sample if it's out of order.
+		return 0, storage.ErrOutOfOrderCT
+	}
+
+	// NOTE: always modify pendingSamples and sampleSeries together.
+	a.pendingSamples = append(a.pendingSamples, record.RefSample{
+		Ref: series.ref,
+		T:   ct,
+		V:   0,
+	})
+	a.sampleSeries = append(a.sampleSeries, series)
+
+	a.metrics.totalAppendedSamples.WithLabelValues(sampleMetricTypeFloat).Inc()
+
+	return storage.SeriesRef(series.ref), nil
 }
 
 // Commit submits the collected samples and purges the batch.
