@@ -15,13 +15,13 @@ package tsdb
 
 import (
 	"context"
+	"errors"
+	"fmt"
+	"log/slog"
 	"math"
 	"os"
 
-	"github.com/go-kit/log"
-	"github.com/go-kit/log/level"
 	"github.com/oklog/ulid"
-	"github.com/pkg/errors"
 
 	"github.com/prometheus/prometheus/model/timestamp"
 	"github.com/prometheus/prometheus/storage"
@@ -30,7 +30,7 @@ import (
 
 // BlockWriter is a block writer that allows appending and flushing series to disk.
 type BlockWriter struct {
-	logger         log.Logger
+	logger         *slog.Logger
 	destinationDir string
 
 	head      *Head
@@ -41,7 +41,7 @@ type BlockWriter struct {
 // ErrNoSeriesAppended is returned if the series count is zero while flushing blocks.
 var ErrNoSeriesAppended = errors.New("no series appended, aborting")
 
-// NewBlockWriter create a new block writer.
+// NewBlockWriter creates a new block writer.
 //
 // The returned writer accumulates all the series in the Head block until `Flush` is called.
 //
@@ -49,7 +49,7 @@ var ErrNoSeriesAppended = errors.New("no series appended, aborting")
 // contains anything at all. It is the caller's responsibility to
 // ensure that the resulting blocks do not overlap etc.
 // Writer ensures the block flush is atomic (via rename).
-func NewBlockWriter(logger log.Logger, dir string, blockSize int64) (*BlockWriter, error) {
+func NewBlockWriter(logger *slog.Logger, dir string, blockSize int64) (*BlockWriter, error) {
 	w := &BlockWriter{
 		logger:         logger,
 		destinationDir: dir,
@@ -65,15 +65,16 @@ func NewBlockWriter(logger log.Logger, dir string, blockSize int64) (*BlockWrite
 func (w *BlockWriter) initHead() error {
 	chunkDir, err := os.MkdirTemp(os.TempDir(), "head")
 	if err != nil {
-		return errors.Wrap(err, "create temp dir")
+		return fmt.Errorf("create temp dir: %w", err)
 	}
 	w.chunkDir = chunkDir
 	opts := DefaultHeadOptions()
 	opts.ChunkRange = w.blockSize
 	opts.ChunkDirRoot = w.chunkDir
+	opts.EnableNativeHistograms.Store(true)
 	h, err := NewHead(nil, w.logger, nil, nil, opts, NewHeadStats())
 	if err != nil {
-		return errors.Wrap(err, "tsdb.NewHead")
+		return fmt.Errorf("tsdb.NewHead: %w", err)
 	}
 
 	w.head = h
@@ -93,7 +94,7 @@ func (w *BlockWriter) Flush(ctx context.Context) (ulid.ULID, error) {
 	// Add +1 millisecond to block maxt because block intervals are half-open: [b.MinTime, b.MaxTime).
 	// Because of this block intervals are always +1 than the total samples it includes.
 	maxt := w.head.MaxTime() + 1
-	level.Info(w.logger).Log("msg", "flushing", "series_count", w.head.NumSeries(), "mint", timestamp.Time(mint), "maxt", timestamp.Time(maxt))
+	w.logger.Info("flushing", "series_count", w.head.NumSeries(), "mint", timestamp.Time(mint), "maxt", timestamp.Time(maxt))
 
 	compactor, err := NewLeveledCompactor(ctx,
 		nil,
@@ -101,20 +102,25 @@ func (w *BlockWriter) Flush(ctx context.Context) (ulid.ULID, error) {
 		[]int64{w.blockSize},
 		chunkenc.NewPool(), nil)
 	if err != nil {
-		return ulid.ULID{}, errors.Wrap(err, "create leveled compactor")
+		return ulid.ULID{}, fmt.Errorf("create leveled compactor: %w", err)
 	}
-	id, err := compactor.Write(w.destinationDir, w.head, mint, maxt, nil)
+	ids, err := compactor.Write(w.destinationDir, w.head, mint, maxt, nil)
 	if err != nil {
-		return ulid.ULID{}, errors.Wrap(err, "compactor write")
+		return ulid.ULID{}, fmt.Errorf("compactor write: %w", err)
 	}
 
-	return id, nil
+	// No block was produced. Caller is responsible to check empty
+	// ulid.ULID based on its use case.
+	if len(ids) == 0 {
+		return ulid.ULID{}, nil
+	}
+	return ids[0], nil
 }
 
 func (w *BlockWriter) Close() error {
 	defer func() {
 		if err := os.RemoveAll(w.chunkDir); err != nil {
-			level.Error(w.logger).Log("msg", "error in deleting BlockWriter files", "err", err)
+			w.logger.Error("error in deleting BlockWriter files", "err", err)
 		}
 	}()
 	return w.head.Close()
