@@ -922,7 +922,7 @@ func (api *API) series(r *http.Request) (result apiFuncResult) {
 			s := q.Select(ctx, true, hints, mset...)
 			sets = append(sets, s)
 		}
-		set = storage.NewMergeSeriesSet(sets, storage.ChainedSeriesMerge)
+		set = storage.NewMergeSeriesSet(sets, 0, storage.ChainedSeriesMerge)
 	} else {
 		// At this point at least one match exists.
 		set = q.Select(ctx, false, hints, matcherSets[0]...)
@@ -1379,7 +1379,7 @@ func (api *API) metricMetadata(r *http.Request) apiFuncResult {
 // RuleDiscovery has info for all rules.
 type RuleDiscovery struct {
 	RuleGroups     []*RuleGroup `json:"groups"`
-	GroupNextToken string       `json:"groupNextToken:omitempty"`
+	GroupNextToken string       `json:"groupNextToken,omitempty"`
 }
 
 // RuleGroup has info for rules which are part of a group.
@@ -1611,7 +1611,7 @@ func parseListRulesPaginationRequest(r *http.Request) (int64, string, *apiFuncRe
 	nextToken := r.URL.Query().Get("group_next_token")
 
 	if nextToken != "" && maxGroups == "" {
-		errResult := invalidParamError(fmt.Errorf("group_limit needs to be present in order to paginate over the groups"), "group_next_token")
+		errResult := invalidParamError(errors.New("group_limit needs to be present in order to paginate over the groups"), "group_next_token")
 		return -1, "", &errResult
 	}
 
@@ -1622,7 +1622,7 @@ func parseListRulesPaginationRequest(r *http.Request) (int64, string, *apiFuncRe
 			return -1, "", &errResult
 		}
 		if parsedMaxGroups <= 0 {
-			errResult := invalidParamError(fmt.Errorf("group_limit needs to be greater than 0"), "group_limit")
+			errResult := invalidParamError(errors.New("group_limit needs to be greater than 0"), "group_limit")
 			return -1, "", &errResult
 		}
 	}
